/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "ErrorDisplayer.h"

#include <QDebug>

ErrorDisplayer::ErrorDisplayer(QWidget* _parent) : QWidget(_parent)
{
    parent = _parent;
    index  = 0;

    layout.setDirection(QBoxLayout::BottomToTop);
    setLayout(&layout);
    layout.setSizeConstraint(QLayout::SetFixedSize);
    reposition();
}

ErrorDisplayer::~ErrorDisplayer()
{
    while (!errors.isEmpty())
    {
        delete errors.takeFirst();
    }
}

void ErrorDisplayer::createError(QString title, QString description,
                                 ErrorType type, int msecDuration)
{
    if (!errors.isEmpty())
    {
        Error* lastError = errors.last();
        if (lastError->compareError(title, description, type))
        {
            lastError->incrementCount(msecDuration);
            return;
        }
    }
    Error* error = new Error(this, title, description, type, msecDuration);
    layout.addWidget(error);
    errors.append(error);
    error->show();
    reposition();
}

void ErrorDisplayer::removeError(Error* error)
{
    layout.removeWidget(error);
    errors.removeAll(error);
    reposition();
}

void ErrorDisplayer::reposition()
{
    adjustSize();
    move(parent->width() - width(), parent->height() - height());
}
