/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "ToggleButton.h"

ToggleButton::ToggleButton(QWidget* parent)
    : QAbstractButton(parent), _switch(false), _opacity(0.000), _height(16),
      _margin(3), _thumb(QColor(213, 213, 213)),
      _anim(new QPropertyAnimation(this, "offset", this))
{
    setOffset(_height / 2);
    _y = _height / 2;
    setBrush(QColor("#009688"));

    connect(this, &QAbstractButton::clicked, this,
            [this]() { emit toggled(_switch); });
}

ToggleButton::ToggleButton(const QBrush& brush, QWidget* parent)
    : QAbstractButton(parent), _switch(false), _opacity(0.000), _height(16),
      _margin(3), _thumb(QColor(213, 213, 213)),
      _anim(new QPropertyAnimation(this, "offset", this))
{
    setOffset(_height / 2);
    _y = _height / 2;
    setBrush(brush);
}

void ToggleButton::paintEvent(QPaintEvent* e)
{
    Q_UNUSED(e);

    QPainter p(this);
    p.setPen(Qt::NoPen);
    if (isEnabled())
    {
        p.setBrush(_switch ? brush() : Qt::black);
        p.setOpacity(_switch ? 0.5 : 0.38);
        p.setRenderHint(QPainter::Antialiasing, true);
        p.drawRoundedRect(QRect(_margin, _margin, width() - 2 * _margin,
                                height() - 2 * _margin),
                          8.0, 8.0);
        p.setBrush(_thumb);
        p.setOpacity(1.0);
        p.drawEllipse(QRectF(offset() - (_height / 2), _y - (_height / 2),
                             height(), height()));
    }
    else
    {
        p.setBrush(Qt::black);
        p.setOpacity(0.12);
        p.drawRoundedRect(QRect(_margin, _margin, width() - 2 * _margin,
                                height() - 2 * _margin),
                          8.0, 8.0);
        p.setOpacity(1.0);
        p.setBrush(QColor("#BDBDBD"));
        p.drawEllipse(QRectF(offset() - (_height / 2), _y - (_height / 2),
                             height(), height()));
    }
}

void ToggleButton::mouseReleaseEvent(QMouseEvent* e)
{
    if (e->button() & Qt::LeftButton)
    {
        changeState(!_switch);
    }
    QAbstractButton::mouseReleaseEvent(e);
}

void ToggleButton::changeState(bool state)
{
    _switch = state;
    _thumb  = _switch ? _brush : QBrush("#d5d5d5");
    if (_switch)
    {
        _anim->setStartValue(_height / 2);
        _anim->setEndValue(width() - _height);
        _anim->setDuration(120);
        _anim->start();
    }
    else
    {
        _anim->setStartValue(offset());
        _anim->setEndValue(_height / 2);
        _anim->setDuration(120);
        _anim->start();
    }
}

void ToggleButton::enterEvent(QEvent* e)
{
    setCursor(Qt::PointingHandCursor);
    QAbstractButton::enterEvent(e);
}

QSize ToggleButton::sizeHint() const
{
    return QSize(2 * (_height + _margin), _height + 2 * _margin);
}
