/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <QMap>
#include <QMenu>
#include <QObject>
#include <QString>

#include "Modules/ModulesList.h"

class Module;
class SaveConfigurationDialog;
class XmlObject;

class ModulesManager : public QObject
{
    Q_OBJECT

public:
    ModulesManager();
    ~ModulesManager();

    static Module *instantiateDefaultModule();
    static Module *invokeModulesPickerService();

    static QList<QString> getModulesNamesList();
    static QString getModuleName(ModuleId id);
    static Module *instantiateModuleById(ModuleId id);

    static Module *instantiateModuleByName(const QString &moduleName);
    static QList<Module *> loadModuleFromXml(XmlObject &xml);
    static QList<ModuleInfo> getModulesInfo();

    void setModules(QList<Module *> modules);
    void addModules(QList<Module *> modules);
    void addPage(Module *module);
    void removePage(Module *module);

    Module *getModuleAt(int index);
    int getModuleCount();
    void clearPages();

    QList<QAction *> getHubMenuActions();

public slots:
    void openNewEmptyWindow();
    void openConfigurationWindow();

signals:
    void contextMenuRequest(QMenu &menu, const QPoint &p);
    void pageAdded(int index);

protected:
    void connectModule(Module *module);
    void disconnectModule(Module *module);

protected slots:
    void onContextMenuRequest(QMenu &menu, const QPoint &p);
    void onModuleDeleted(Module *deletedModule);
    void onReplaceMeWith(Module *sender, Module *newModule);

private:
    static ModulesList modulesListHandler;

    QList<Module *> pages;
    QList<QAction *> menuActions;
};
