/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "CommandPad.h"

#include <Modules/CommandPad/MessagesList.h>

#include <QVBoxLayout>

CommandPad::CommandPad(QWidget *parent) : DefaultModule(parent)
{
    setupUi();
    defaultContextMenuSetup();
}

CommandPad::~CommandPad()
{
    delete messagesListComboBox;
    for (auto key : formElements.keys())
        delete formElements[key];
}

QWidget *CommandPad::toWidget() { return this; }

XmlObject CommandPad::toXmlObject()
{
    XmlObject obj = XmlObject(getName(ModuleId::COMMANDPAD));

    auto key = messagesListComboBox->currentText();
    if (formElements.contains(key))
    {
        obj.addAttribute("message_id", key);
        formElements[key]->toXmlObject(obj);
    }

    return obj;
}

void CommandPad::fromXmlObject(const XmlObject &obj)
{
    if (obj.getObjectName() == getName(ModuleId::COMMANDPAD))
    {
        auto key = obj.getAttribute("message_id");

        if (formElements.contains(key))
        {
            formElements[key]->fromXmlObject(obj);
            messagesListComboBox->setCurrentText(key);
        }
    }
}

void CommandPad::setupUi()
{
    QVBoxLayout *outerLayout = new QVBoxLayout;
    outerLayout->addStretch();

    messagesListComboBox = new QComboBox;
    messagesListComboBox->setSizePolicy(
        QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
    messagesListComboBox->addItems(MessagesList::messagesList);
    outerLayout->addWidget(messagesListComboBox);

    QGridLayout *formGridLayout = new QGridLayout;
    outerLayout->addLayout(formGridLayout);

    QPushButton *sendButton = new QPushButton("Send");
    outerLayout->addWidget(sendButton);

    setLayout(outerLayout);

    MessagesList::fillMessagesMap(formElements);

    // Set default value
    currentMessage = "PING_TC";
    messagesListComboBox->setCurrentText(currentMessage);

    connect(messagesListComboBox, &QComboBox::currentTextChanged, this,
            [=](QString key)
            {
                if (formElements.contains(key))
                {
                    formElements[currentMessage]->removeFromGridLayout(
                        formGridLayout);
                    formElements[key]->applyToGridLayout(formGridLayout);
                    currentMessage = key;
                }
            });

    connect(sendButton, &QPushButton::clicked,
            [=]()
            {
                auto key = messagesListComboBox->currentText();
                if (formElements.contains(key))
                {
                    auto message = formElements[key]->prepareMessage(key);
                    getCore()->getMessageBroker()->publish(message);
                }
            });
}
