/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "SkywardHubMainWindow.h"

#include <Core/Module/Module.h>
#include <Core/ModulesManager/ModulesManager.h>

#include <QFile>
#include <QMenu>

SkywardHubMainWindow::SkywardHubMainWindow(QWidget* parent)
    : QMainWindow(parent)
{
    QFile styleSheet(":/assets/styles/global.qss");
    styleSheet.open(QFile::ReadOnly);
    setStyleSheet(styleSheet.readAll());

    mainWindow = new Window();
    mainWindow->setWindowTitle("Skyward Hub - 1");
    setCentralWidget(mainWindow);

    connect(core->getModulesManager(), &ModulesManager::contextMenuRequest,
            this, &SkywardHubMainWindow::onContextMenuRequest);
    connect(core->getModulesManager(), &ModulesManager::pageAdded, this,
            &SkywardHubMainWindow::onPageAdded);

    core->init();
}

void SkywardHubMainWindow::onContextMenuRequest(QMenu& menu, const QPoint& pos)
{
    menu.exec(pos);
}

void SkywardHubMainWindow::onPageAdded(int index)
{
    Module* module = core->getModulesManager()->getModuleAt(index);

    if (module != nullptr)
    {
        if (index == 0)
        {
            mainWindow->updateModule(module);
        }
        else
        {
            Window* window = new Window();
            window->setWindowTitle("Skyward Hub - " +
                                   QString::number(index + 1));
            window->setCentralModule(module);
            window->show();
        }
    }
}
