/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "SerialMavlinkModule.h"

#include <QSerialPortInfo>

SerialMavlinkModule::SerialMavlinkModule(QWidget *parent)
    : BaseMavlinkModule(&serial, parent), serial(this)
{
    childSetupUi();
}

void SerialMavlinkModule::onRefreshClicked() { fillPortsComboBox(); }

XmlObject SerialMavlinkModule::childToXmlObject()
{
    XmlObject obj(getName(ModuleId::SERIAL_MAVLINK));
    obj.addAttribute("serial_port", portsComboBox->currentText());
    obj.addAttribute("baudrate", baudrateComboBox->currentData().toInt());

    return obj;
}

void SerialMavlinkModule::childFromXmlObject(const XmlObject &obj)
{
    QString serialPort = obj.getAttribute("serial_port");
    int baudrate;
    obj.getAttribute("baudrate", baudrate);

    portsComboBox->setCurrentText(serialPort);
    baudrateComboBox->setCurrentText(QString::number(baudrate));
}

void SerialMavlinkModule::childSetupUi()
{
    QLabel *portsLabel = new QLabel("Available ports:");
    portsLabel->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    childLayout->addWidget(portsLabel);

    portsComboBox = new QComboBox;
    portsComboBox->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    childLayout->addWidget(portsComboBox);

    fillPortsComboBox();

    refreshButton = new QPushButton("Refresh");
    refreshButton->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    connect(refreshButton, &QPushButton::released, this,
            &SerialMavlinkModule::onRefreshClicked);
    childLayout->addWidget(refreshButton);

    QLabel *baudrateLabel = new QLabel("Baudrate:");
    baudrateLabel->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    childLayout->addWidget(baudrateLabel);

    baudrateComboBox = new QComboBox;
    baudrateComboBox->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));

    baudrateComboBox->addItem("115200", QSerialPort::Baud115200);
    baudrateComboBox->addItem("19200", QSerialPort::Baud19200);
    baudrateComboBox->addItem("9600", QSerialPort::Baud9600);

    childLayout->addWidget(baudrateComboBox);
}

void SerialMavlinkModule::childDisableControls()
{
    portsComboBox->setEnabled(false);
    baudrateComboBox->setEnabled(false);
    refreshButton->setEnabled(false);
}

void SerialMavlinkModule::childEnableControls()
{
    portsComboBox->setEnabled(true);
    baudrateComboBox->setEnabled(true);
    refreshButton->setEnabled(true);
}

bool SerialMavlinkModule::open()
{
    QString portName = portsComboBox->currentText();
    int baudRate     = baudrateComboBox->currentData().toInt();

    // Open the serial port
    return serial.open(portName, baudRate);
}

void SerialMavlinkModule::fillPortsComboBox()
{
    portsComboBox->clear();

    const auto serialPortInfos = QSerialPortInfo::availablePorts();
    for (auto serialPortInfo : serialPortInfos)
        portsComboBox->addItem(serialPortInfo.portName());
}