/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "UdpMavlinkModule.h"

UdpMavlinkModule::UdpMavlinkModule(QWidget *parent)
    : BaseMavlinkModule(&udp, parent), udp(this)
{
    childSetupUi();
}

XmlObject UdpMavlinkModule::childToXmlObject()
{
    XmlObject obj(getName(ModuleId::UDP_MAVLINK));
    obj.addAttribute("recv_port", recvPort->text().toInt());
    obj.addAttribute("send_port", sendPort->text().toInt());

    return obj;
}

void UdpMavlinkModule::childFromXmlObject(const XmlObject &obj)
{
    QString serialPort = obj.getAttribute("serial_port");
    int recvPortValue, sendPortValue;
    obj.getAttribute("recv_port", recvPortValue);
    obj.getAttribute("send_port", sendPortValue);

    recvPort->setText(QString::number(recvPortValue));
    sendPort->setText(QString::number(sendPortValue));
}

void UdpMavlinkModule::childSetupUi()
{
    QLabel *recvPortLabel = new QLabel("Receive port:");
    recvPortLabel->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    childLayout->addWidget(recvPortLabel);

    recvPort = new QLineEdit;
    recvPort->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    recvPort->setValidator(new QIntValidator(0, 0xffff));
    childLayout->addWidget(recvPort);

    QLabel *sendPortLabel = new QLabel("Send port:");
    sendPortLabel->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    childLayout->addWidget(sendPortLabel);

    sendPort = new QLineEdit;
    sendPort->setSizePolicy(
        QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    sendPort->setValidator(new QIntValidator(0, 0xffff));
    childLayout->addWidget(sendPort);
}

void UdpMavlinkModule::childDisableControls()
{
    recvPort->setEnabled(false);
    sendPort->setEnabled(false);
}

void UdpMavlinkModule::childEnableControls()
{
    recvPort->setEnabled(true);
    sendPort->setEnabled(true);
}

bool UdpMavlinkModule::open()
{
    int recvPortValue, sendPortValue;
    bool recvPortOk, sendPortOk;

    recvPortValue = recvPort->text().toInt(&recvPortOk);
    sendPortValue = sendPort->text().toInt(&sendPortOk);

    // Check if the parameters are ok
    if (!recvPortValue || !sendPortValue)
        return false;

    // Open the serial port
    return udp.open(recvPortValue, sendPortValue);
}