/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "ModuleInfo.h"

ModuleInfo::ModuleInfo() {}

ModuleInfo::ModuleInfo(const ModuleId &id, const QString &name)
{
    setId(id);
    setModuleName(name);
}

ModuleInfo::ModuleInfo(const ModuleInfo &other) { copy(other); }

ModuleInfo::ModuleInfo(const ModuleId &id, const QString &name,
                       ModuleCategory category)
    : ModuleInfo(id, name)
{
    setCategory(category);
}

ModuleId ModuleInfo::getId() const { return moduleId; }

void ModuleInfo::setId(const ModuleId &value) { moduleId = value; }

QString ModuleInfo::getModuleName() const { return moduleName; }

void ModuleInfo::setModuleName(const QString &value) { moduleName = value; }

std::function<Module *()> ModuleInfo::getFactory() const { return factory; }

void ModuleInfo::setFactory(const std::function<Module *()> &value)
{
    factory = value;
}

QStringList ModuleInfo::getModuleSourceFiles() const
{
    return moduleSourceFiles;
}

void ModuleInfo::addModuleSourceFiles(const QString &value)
{
    moduleSourceFiles.append(value);
}

void ModuleInfo::operator=(const ModuleInfo &other) { copy(other); }

void ModuleInfo::copy(const ModuleInfo &other)
{
    setId(other.getId());
    setCategory(other.getModuleCategory());
    setModuleName(other.getModuleName());
    setFactory(other.getFactory());
    moduleSourceFiles = other.getModuleSourceFiles();
}

ModuleCategory ModuleInfo::getModuleCategory() const { return mCategory; }

void ModuleInfo::setCategory(const ModuleCategory &value) { mCategory = value; }
