/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <QRadioButton>
#include <QWidget>

#include "ValueElement.h"

namespace Ui
{
class ValuesViewerConfigPanel;
}

class ValuesViewerConfigPanel : public QWidget
{
    Q_OBJECT

public:
    explicit ValuesViewerConfigPanel(QWidget *parent = nullptr);
    ~ValuesViewerConfigPanel();

    int getColumnsCount() const;
    void setColumnsCount(int value);

    void setRules(const QList<ValueElement> &newValues);
    QList<ValueElement> getRules() const;

signals:
    void configurationSaved(ValuesViewerConfigPanel *sPanel);

protected:
    void onAddFieldClicked();
    void onRemoveClicked();
    void onPickColordClicked();
    void onValueElementClicked();
    void onSaveClicked();
    void onEditClicked();
    int computeViewIndex(const QString &viewName);
    QRadioButton *createView(const ValueElement &el);
    void setTextBoxValues(const ValueElement &el);
    int findSelectedView() const;
    void addRule(int index, const ValueElement &el);
    void onEmitRadioButtonToggled(bool val);

    ValueElement createRuleFromControls();

private:
    Ui::ValuesViewerConfigPanel *ui;

    QList<ValueElement> rules;
    QList<QRadioButton *> views;
};
